package dluploadclient;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.io.File;
import java.util.Stack;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import net.iharder.dnd.FileDrop;



/**
 * Interface grahique de l'application Client dl.free.fr
 * 
 * <p>
 * <code>
 * Copyright 2008, Maurer Gilles
 * </code>
 * <p>
 * <code>
 * This file is part of Client dl.free.fr.
 * </code>
 * <p>
 * <code>
 * Client dl.free.fr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * </code>
 *<p>
 * <code>
 * Client dl.free.fr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * </code>
 * <p>
 * <code>
 * You should have received a copy of the GNU General Public License
 * along with Client dl.free.fr. If not, see <http://www.gnu.org/licenses/>.
 * </code>
 * 
 * @author Maurer Gilles
 *
 */
public class DLUploadClientGUI extends JFrame
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Affichage du text "Adresse e-mail :"
	 */
	private JLabel jLblMail;
	
	/**
	 * Affichage du text "Fichier courant :"
	 */
	private JLabel jLblCurrentFile;
	
	/**
	 * Affichage du text "Total :"
	 */
	private JLabel jLblAllFile;
	
	/**
	 * Affichage du text "Mot de passe :"
	 */
	private JLabel jLblPassword;

	/**
	 * Bouton d'upload
	 */
	private JButton jBtnUpload;
	
	/**
	 * Bouton pour annuler un upload
	 */
	private JButton jBtnCancel;
	
	/**
	 * Bouton pour tuer un upload --> non fonctionnel.
	 */
	//private JButton jBtnKill;

	/**
	 * Bouton d'ajout de fichier
	 */
	private JButton jBtnAddFile;

	/**
	 * Bouton de supression de fichiers
	 */
	private JButton jBtnDelFile;
	
	/**
	 * Bouton d'affichage des URL
	 */
	private JButton jBtnViewURL;

	
	/**
	 * Champ de texte de l'adresse e-mail
	 */
	private JTextField jtxtMail;

	/**
	 * Champ de texte du mot de passe
	 */
	private JPasswordField jtxtPassword;
	
	/**
	 * Case  cocher pour cacher le mot de passe
	 */
	private JCheckBox jChkBoxPassword;

	/**
	 * Champ de texte Multi Ligne pour le log FTP
	 */
	private JTextArea jtxtLogFTP;

	/**
	 * Panneau de dfilement pour le log FTP
	 */
	private JScrollPane jscrollLogFTP;

	/**
	 * Tableau pour l'affichage de la liste des fichiers
	 */
	private JTable jtableFileList;

	/**
	 * Conteneur de la liste des fichiers  uploader
	 */
	private Vector<UploadFile> vectFileList;
	
	/**
	 * Panneau de dfilement pour le tableau des fichiers
	 */
	private JScrollPane jFilescrollPane;
	
	/**
	 * Boolen pour savoir si un lment est slectionn dans la liste de fichiers
	 */
	private boolean jtableFileListSomethinSelected;
	

	/**
	 * Encadr pour la gestion de fichiers
	 */
	private JPanel jPnlFileManager;
	
	/**
	 * Encadr pour le Log FTP
	 */
	private JPanel jPnlFTPLog;
	
	/**
	 * Encadr pour la gestion de l'identificaton
	 */
	private JPanel jPnlLoginManager;
	
	/**
	 * Encadr pour l'affichage de message d'erreur
	 */
	private JPanel jPnlErrorList;
	
	// Thread d'upload
	/**
	 * Unit d'excution en charge de l'upload des fichiers
	 */
	private ThreadUpload trdUpload;
	
	/**
	 * Barre de progression du fichier courant
	 */
	private JProgressBar jprgbarCurrentUpload;
	
	/**
	 * Barre de progression du fichier globale
	 */
	private JProgressBar jprgbarGlobal;
	
	/**
	 * Taille dtecte comme dja uploade
	 */
	private long restartSize;
	
	/**
	 * Reprsente l'tat actuelle de l'application
	 */
	private int state;
	
	/**
	 * Ecouteur pour le glisser dposer de fichier sur l'application
	 */
	private FileDrop fileDrop;
	
	/**
	 * Etat de l'application : En attente
	 */
	public static final int WAITING = 0;
	
	/**
	 * Etat de l'application : En cours d'upload
	 */
	public static final int UPLOADING = 1;
	
	/**
	 * Etat de l'application : Upload termin
	 */
	public static final int UPLOADED = 2;
	
	/**
	 * Etat de l'application : Upload termin
	 */
	public static final int ADDINGFILES = 3;
	
	/**
	 * Etat de l'application : Annulation en cours
	 */
	public static final int CANCELLINGTRANSFER = 3;
	
	/**
	 * Marge verticale minimale des composants
	 */
	public static final int VERTICAL_PADDING = 0;

	/**
	 * Marge horizontale minimale des composants
	 */
	public static final int HORIZONTAL_PADDING = 0;

	/**
	 * Poids vertical de la colonne : colonne normale
	 */
	public static final double WEIGHTY_NORMAL = 0.01;

	/**
	 * Poids horizontal de la colonne : colonne en bordure
	 */
	public static final double WEIGHTX_BORDER = 0.05;
	
	/**
	 * Poids vertical de la colonne : colonne du Log
	 */
	public static final double WEIGHTY_LOG = 0.8;
	
	/**
	 * Poids horizontal de la colonne : colonne centrale
	 */
	public static final double WEIGHTX_CENTER = 0.9;
	
	/**
	 * Titre de la fentre
	 */
	public static final String WINDOWTITLE = "Client dl.free.fr";
	
	
	/**
	 * Constructeur de la classe de l'interface utilisateur
	 */
	public DLUploadClientGUI()
	{
		// TODO : rendre le mot de passe optionnel
		// TODO : Colorer le log FTP
		// TODO : supporter la protection par mot de passe pour le tlchargement
		// TODO : vrifier / corriger l'tat des boutons (pas de fichier a charger = envoi tout de mme actif)
		// TODO : crer des listener pour grer la prsence ou non d'un nom d'utilisateur etc
		// TODO : Quand on annule : la barre de progression totale revient pas  zro, alors que lorsqu'on recommence, elle revient  roo, c'est pas cohrent
		// TODO : voir s'il parait cohrent de filtrer certain fichier lors de l'ajout de dossier / drag & drop :  Desktop.ini et Thumbs.db lors du drag & drop d'un rpertoire, ainsi que les .DS_Store et .localized (Mac).
		// TODO : mettre en place les bulles d'aide
		// TODO : rajouter le message d'erreur durant l'upload dans le log FT tant qu'a faire !
		
		// On paramtre le titre de la fentre
		this.setTitle(DLUploadClientGUI.WINDOWTITLE);
		
		// Cration des composants
		this.jLblMail = new JLabel("Adresse e-mail :");
		this.jLblPassword = new JLabel("Mot de passe :");
		this.jLblCurrentFile= new JLabel("Fichier courant :");
		this.jLblAllFile = new JLabel("Total :");
		this.jBtnUpload = new JButton("Envoyer");
		this.jtxtMail = new JTextField(20);
		this.jtxtPassword = new JPasswordField(20);
		this.jtxtLogFTP = new JTextArea(10, 10);
		this.jscrollLogFTP = new JScrollPane(this.jtxtLogFTP);
		this.jBtnCancel = new JButton("Annuler");
		this.jBtnAddFile = new JButton("Ajouter");
		this.jBtnDelFile = new JButton("Supprimer");
		this.jBtnViewURL = new JButton("Liste des liens");
		this.jChkBoxPassword = new JCheckBox("Cacher", true);
		
		//this.jBtnKill = new JButton("Stopper");
		
		
		/* 
		 * Cration du modle de la liste des fichiers
		 */
		this.vectFileList = new Vector<UploadFile>();
		Vector<String> columnNames = new Vector<String>();
		columnNames.add("Fichier");
		columnNames.add("Etat");
		
		FileListTableModel jFileListTableModel = new FileListTableModel(columnNames, this.vectFileList);
		this.jtableFileList = new JTable(jFileListTableModel);

		// On active la selection par ligne entire
	    this.jtableFileList.setColumnSelectionAllowed(false);
	    this.jtableFileList.setRowSelectionAllowed(true);
	
		// Activation des paramtres du Log FTP
		this.jtxtLogFTP.setEditable(false);
		this.jtxtLogFTP.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 12));
		
		
		
		// On cre les barre de progresion 
		this.jprgbarCurrentUpload = new JProgressBar(0, 100);
		this.jprgbarCurrentUpload.setStringPainted(true);
		
		this.jprgbarGlobal = new JProgressBar(0, 100);
		this.jprgbarGlobal.setStringPainted(true);
		
		// Ajout des vnements
		//this.jBtnChooseFile.addActionListener(new JBtnChooseFileListener(this));
		this.jBtnUpload.addActionListener(new JBtnUploadListener(this));
		this.jBtnCancel.addActionListener(new JBtnCancelUploadListener(this));
		//this.jBtnKill.addActionListener(new JBtnKillUploadListener(this));
		//this.jBtnCopyToClipboard.addActionListener(new JBtnCopyToClipboardListener(this));
		this.addWindowListener(new DLUploadWindowAdapter(this));
		this.jChkBoxPassword.addActionListener(new JChkBoxPasswordListener(this));
		this.jBtnAddFile.addActionListener(new JBtnAddFileListener(this));
		this.jBtnDelFile.addActionListener(new JBtnDelFileListener(this));
		
		JTableFileListListSelectionListener listSelectionListener = new JTableFileListListSelectionListener(this.jtableFileList, this);
		this.jtableFileList.getSelectionModel().addListSelectionListener(listSelectionListener);
		this.jtableFileList.getColumnModel().getSelectionModel().addListSelectionListener(listSelectionListener);
		this.jBtnViewURL.addActionListener(new JBtnViewURLListener(this));
		
		// Ajout des layouts
		
		Container contentPane = this.getContentPane();
		GridBagLayout backgroundLayout = new GridBagLayout();
		contentPane.setLayout(backgroundLayout);
		
		GridBagLayout fileListLayout = new GridBagLayout();
		GridBagLayout FTPLogLayout = new GridBagLayout();
		GridBagLayout loginManagerLayout = new GridBagLayout();
		GridBagLayout errorListLayout = new GridBagLayout();
		
		
		
		// On va crer les panels ncessaire
		
		this.jPnlFileManager = new JPanel(fileListLayout);
		this.jPnlLoginManager = new JPanel(loginManagerLayout);
		this.jPnlFTPLog = new JPanel(FTPLogLayout);
		this.jPnlErrorList = new JPanel(errorListLayout);
		
		this.jPnlFileManager.setBorder(BorderFactory.createTitledBorder("Fichiers"));
		this.jPnlFTPLog.setBorder(BorderFactory.createTitledBorder("Log FTP"));
		this.jPnlLoginManager.setBorder(BorderFactory.createTitledBorder("Identification"));
		this.jPnlErrorList.setBorder(BorderFactory.createTitledBorder("Erreurs"));
		
		/*
		 * Ajout des panel principaux
		 */
		Insets globalInsets = new Insets(5, 5, 5, 5);
		
		GridBagConstraints constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 1;
		constraint.weighty = 1;
		constraint.fill = GridBagConstraints.BOTH;
		contentPane.add(this.jPnlFileManager, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 1;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.HORIZONTAL;
		contentPane.add(this.jPnlLoginManager, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 3;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 1;
		constraint.weighty = 1;
		constraint.fill = GridBagConstraints.BOTH;
		contentPane.add(this.jPnlFTPLog, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 2;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 1;
		constraint.weighty = 1;
		constraint.fill = GridBagConstraints.BOTH;
		contentPane.add(this.jPnlErrorList, constraint);
		
		
		/*
		 * Ici, on ajoute le contenu du panel de la liste des fichiers
		 */	
		
		// On cre un panel pour dfiler en cas de ncessit
        this.jFilescrollPane = new JScrollPane(jtableFileList);
        this.jFilescrollPane.setMinimumSize(new Dimension(10,80));
        this.jFilescrollPane.setPreferredSize(new Dimension(30,80));
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_BORDER;
		constraint.weighty = DLUploadClientGUI.WEIGHTY_NORMAL;
		constraint.gridwidth = 3;
		constraint.fill = GridBagConstraints.BOTH;
		this.jPnlFileManager.add(this.jFilescrollPane, constraint);
		
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 2;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_CENTER;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlFileManager.add(jBtnAddFile, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 1;
		constraint.gridy = 2;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_CENTER;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlFileManager.add(jBtnDelFile, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 2;
		constraint.gridy = 2;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_CENTER;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlFileManager.add(this.jBtnViewURL, constraint);
		
		
		/*
		 * Ici on ajoute le contenu du panel du log FTP
		 */
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_BORDER;
		constraint.weighty = DLUploadClientGUI.WEIGHTY_NORMAL;
		constraint.fill = GridBagConstraints.BOTH;
		
		this.jPnlFTPLog.add(this.jscrollLogFTP, constraint);
		
		/*
		 * Ici on ajoute le contenu du panel de l'identification
		 */
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.WEST;
		constraint.gridx = 0;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jLblMail, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 1;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_BORDER;
		constraint.weighty = DLUploadClientGUI.WEIGHTY_NORMAL;
		constraint.fill = GridBagConstraints.HORIZONTAL;
		this.jPnlLoginManager.add(this.jtxtMail, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.WEST;
		constraint.gridx = 0;
		constraint.gridy = 1;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jLblPassword, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 1;
		constraint.gridy = 1;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_BORDER;
		constraint.weighty = DLUploadClientGUI.WEIGHTY_NORMAL;
		constraint.fill = GridBagConstraints.HORIZONTAL;
		this.jPnlLoginManager.add(this.jtxtPassword, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.EAST;
		constraint.gridx = 2;
		constraint.gridy = 1;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jChkBoxPassword, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.WEST;
		constraint.gridx = 0;
		constraint.gridy = 3;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jBtnUpload, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.EAST;
		constraint.gridx = 2;
		constraint.gridy = 3;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jBtnCancel, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.EAST;
		constraint.gridx = 1;
		constraint.gridy = 4;
		constraint.gridwidth = GridBagConstraints.REMAINDER;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 1;
		constraint.weighty = 1;
		constraint.fill = GridBagConstraints.BOTH;
		this.jPnlLoginManager.add(this.jprgbarCurrentUpload, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.WEST;
		constraint.gridx = 0;
		constraint.gridy = 4;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jLblCurrentFile, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.EAST;
		constraint.gridx = 1;
		constraint.gridy = 5;
		constraint.gridwidth = GridBagConstraints.REMAINDER;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 1;
		constraint.weighty = 1;
		constraint.fill = GridBagConstraints.BOTH;
		this.jPnlLoginManager.add(this.jprgbarGlobal, constraint);
		
		constraint = new GridBagConstraints();
		constraint.anchor = GridBagConstraints.WEST;
		constraint.gridx = 0;
		constraint.gridy = 5;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		this.jPnlLoginManager.add(this.jLblAllFile, constraint);
		
		// Gestion du glisser dposer
		
		FileDrop.Listener fileDropListener = new DLUploadFileDropListener(this);
		
		this.fileDrop = new FileDrop(System.out, this, true, fileDropListener);
		
		
		// Mise  jour de l'tat de l'application
		this.setApplicationState(DLUploadClientGUI.WAITING);
		this.launchEventAndStatusManager();
		
		// On rduit la fentre  sa taille minimale, et on l'utilise en tant que taille minimale de l'application
		this.pack();
		this.setMinimumSize(this.getSize());
		this.setLocationRelativeTo(null);
	}

	
	
	/**
	 * Retourne un objet String contenant l'actuel adresse e-mail.
	 * @return l'adresse e-mail actuelle
	 */
	public String getMail()
	{
		return this.jtxtMail.getText();
	}

	
	/**
	 * Retourne un objet String contenant l'actuel mot de passe.
	 * @return le mot de passe actuel
	 */
	public String getPassword()
	{
		return new String(this.jtxtPassword.getPassword());
	}
	
	/**
	 * Retour un objet ThreadUpload qui est l'unit d'excution qui gre l'upload des fichiers
	 * @return le thread d'upload
	 */
	public ThreadUpload getTrdUpload()
	{
		return this.trdUpload;
	}
	
	/**
	 * Modifie le pourcentage de progression de la barre de progression
	 * @param value Valeur dcimale de 0  100
	 */
	public void setCurrentUploadProgressBarValue(double value)
	{
		this.jprgbarCurrentUpload.setValue((int)value);
		this.jprgbarCurrentUpload.setString(String.format("%.2f %%", new Double(value)));
	}
	
	public void setGlobalProgressBarValue(double value)
	{
		this.jprgbarGlobal.setValue((int)value);
		this.jprgbarGlobal.setString(String.format("%.2f %%", new Double(value)));
	}
	
	/*public void setState(int state)
	{
		this.state = state;
	}*/
	
	
	/**
	 * Permet de mettre  jour la rfrence vers l'unit d'excution d'upload de fichier.
	 * @param trdUpload La rfrence vers la nouvelle unit d'excution
	 */
	public void setTrdUpload(ThreadUpload trdUpload)
	{
		this.trdUpload = trdUpload;
	}
	
		
	/**
	 * Permet de mettre  jour l'octet de dbut de transfert pour le fichier en cours.
	 * @param size octet de dbut du transfert
	 */
	public void setRestartSize(long size)
	{
		this.restartSize = size;
	}
	
	/**
	 * Renvoi un nombre en type natif long qui contient l'octet de dbut du transfert du fichier en cours.
	 * @return octet de dbut du transfert du fichier courant
	 */
	public long getRestartSize()
	{
		return this.restartSize;
	}
	
	
	/**
	 * Ajout une ligne  la fin du log FTP.
	 * @param line La ligne  ajouter
	 */
	public void addLogFTPLine(String line)
	{
		this.jtxtLogFTP.setText(this.jtxtLogFTP.getText() + line);
	}
	
	/**
	 * Retourne un object du type JTextArea pour accder  l'objet contenant le log FTP
	 * @return un object du type JTextArea pour accder  l'objet contenant le log FTP
	 */
	public JTextArea getjtxtLogFTP()
	{
		return this.jtxtLogFTP;
	}
	
	/**
	 * Retourne un objet String qui contient une version compatible avec dl.free.fr du nom de fichier entr
	 * @param name Le mom de fichier  rendre compatible
	 * @return Le nom de fichier rendu compatible
	 */
	public static String sanitizeFilename(String name)
	{
		return name.replaceAll("[^a-zA-Z0-9\\.\\(\\)\\-\\_\\[\\]]", "");
	}
	
	/**
	 * Ecrit le contenu du paramtre writeMe dans le presse papier
	 * @param writeMe Le texte  mettre dans le presse papier
	 */
	public static void writeToClipboard(String writeMe)
	{
		// get the system clipboard
		Clipboard systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		// set the textual content on the clipboard to our 
		// transferable object
		// we use the 
		Transferable transferableText = new StringSelection(writeMe);
		systemClipboard.setContents(transferableText,null);
	}
	
	
	/**
	 * Retourne un int correspondant au statut de l'application
	 * @return Le statut actuel de l'application
	 */
	public int getApplicationState()
	{
		return this.state;
	}
	
	/**
	 * Met  jour le statut de l'application
	 * @param state Statut de l'application  appliquer
	 */
	public void setApplicationState(int state)
	{
		this.state = state;
	}


	/**
	 * Retoure une rfrence  la case  cocher pour afficher / cacher le mot de passe sous forme d'un objet de type JCheckBox.
	 * @return Une rfrence vers la case  cocher 
	 */
	public JCheckBox getJChkBoxPassword()
	{
		return jChkBoxPassword;
	}


	/**
	 * Retourne une rfrence au champ de texte contenant le mot de passe.
	 * @return Une rfrence vers le champ de texte
	 */
	public JPasswordField getJtxtPassword() {
		return jtxtPassword;
	}
	
	/**
	 * Permet d'ajouter des fichiers avec un tableau d'objet de type File.
	 * @param files Le tableau de fichiers  ajouter
	 */
	public void addFile(File[] files)
	{


		Stack<File> fileList = new Stack<File>();
		
		Vector<UploadFile> uploadFileList = new Vector<UploadFile>();
		
		for (File file : files)
		{
			fileList.add(file);
		}
		
		File currentFile = null;
		File[] directoryContent = null;
		
		// Tant que la liste n'est pas vide
		while(!fileList.isEmpty())
		{
			currentFile = fileList.pop();
			
			if(currentFile.isFile())
			{
				// Si c'est un fichier, on l'ajoute  la liste des fichiers
				uploadFileList.add(new UploadFile(currentFile.getAbsolutePath()));
			}
			else if(currentFile.isDirectory())
			{
				// Si c'est un dossier, on liste le contenu
				directoryContent = currentFile.listFiles();
				
				// Et on ajoute le contenu  la pile
				for (File tempFile : directoryContent)
				{
					fileList.push(tempFile);
				}
				
			}
			
		}
		
		// On ajoute tout les fichiers  la liste des fichiers
		this.vectFileList.addAll(uploadFileList);
		
		// On force la mise  jour de l'interface
		this.jtableFileList.addNotify();
	}


	/**
	 * Retourne 
	 * @return true si quelquechose est slectionn dans la liste des fichiers, false sinon 
	 */
	public boolean isJtableFileListSomethinSelected()
	{
		return jtableFileListSomethinSelected;
	}


	/**
	 * Met  jour la variable qui undique si quelquechose a t slectionn dans la table des fichiers 
	 * @param jtableFileListSomethinSelected vrai si quelquechose est slectionn dans la table des fichiers, false sinon
	 */
	public void setJtableFileListSomethinSelected(boolean jtableFileListSomethinSelected)
	{
		this.jtableFileListSomethinSelected = jtableFileListSomethinSelected;
	}
	
	
	/**
	 * Fonction qui va mettre  jour le statut de tous les composants selon l'tat actuel de l'application.
	 */

	public void launchEventAndStatusManager()
	{
		// Dans le cas ou l'application est au repos
		// Actif : Ajouter, Liste de liens, Envoyer, Mail, Mot de passe
		if(this.getApplicationState() == DLUploadClientGUI.WAITING || this.getApplicationState() == DLUploadClientGUI.UPLOADED)
		{
			this.jBtnAddFile.setEnabled(true);
			this.jBtnDelFile.setEnabled(this.isJtableFileListSomethinSelected());
			this.jBtnCancel.setEnabled(false);
			this.jBtnUpload.setEnabled(true);
			this.jtableFileList.setEnabled(true);
			this.jtxtMail.setEnabled(true);
			this.jtxtPassword.setEnabled(true);
			this.jChkBoxPassword.setEnabled(true);
			this.jBtnViewURL.setEnabled(true);
		}
		// Dans le cas o l'application attends l'ajout de fichiers ou annule une action
		// (Rien n'est actif)
		else if(this.getApplicationState() == DLUploadClientGUI.ADDINGFILES || this.getApplicationState() == DLUploadClientGUI.CANCELLINGTRANSFER)
		{
			this.jBtnAddFile.setEnabled(false);
			this.jBtnDelFile.setEnabled(false);
			this.jBtnCancel.setEnabled(false);
			this.jBtnUpload.setEnabled(false);
			this.jtableFileList.setEnabled(false);
			this.jtxtMail.setEnabled(false);
			this.jtxtPassword.setEnabled(false);
			this.jChkBoxPassword.setEnabled(false);
			this.jBtnViewURL.setEnabled(false);
		}
		// Dans le cas o l'application est en train d'uploader
		// (Annuler est actif)
		else if(this.getApplicationState() == DLUploadClientGUI.UPLOADING)
		{
			this.jBtnAddFile.setEnabled(false);
			this.jBtnDelFile.setEnabled(false);
			this.jBtnCancel.setEnabled(true);
			this.jBtnUpload.setEnabled(false);
			this.jtableFileList.setEnabled(false);
			this.jtxtMail.setEnabled(false);
			this.jtxtPassword.setEnabled(false);
			this.jChkBoxPassword.setEnabled(false);
			this.jBtnViewURL.setEnabled(false);
		}
		
	}


	/**
	 * Retourne la liste des fichiers ajout  l'application
	 * @return La liste des fichiers
	 */
	public Vector<UploadFile> getVectFileList()
	{
		return this.vectFileList;
	}
	
	/**
	 * Permet de notifier  la table de fichier une modification de son contenu
	 */
	public void jtableFileListRefresh()
	{
		this.jtableFileList.revalidate();
		this.jtableFileList.repaint();
		this.jFilescrollPane.revalidate();
	}


	/**
	 * Rertoune une rfrence au composant qui permet d'afficher le contenu de la liste des fichiers.
	 * @return une rfrence  la table de fichier
	 */
	public JTable getJtableFileList()
	{
		return jtableFileList;
	}
	
	/**
	 * Retourne un objet de type {@link Vector} contenant la liste des fichiers non encore upload
	 * @return la liste des fichiers non encore upload
	 */
	public Vector<UploadFile> getVectFileListToUpload()
	{
		Vector<UploadFile> fileList = new Vector<UploadFile>(); 
		for (UploadFile currentFile : this.vectFileList)
		{
			if(currentFile.getState() != UploadFile.UPLOADED)
			{
				fileList.add(currentFile);
			}
			
		}
		
		return fileList;
	}
	
	/**
	 * Retourne la liste des fichiers dja upload
	 * @return la liste des fichiers dj upload
	 */
	public Vector<UploadFile> getVectFileListUploaded()
	{
		Vector<UploadFile> fileList = new Vector<UploadFile>(); 
		for (UploadFile currentFile : this.vectFileList)
		{
			if(currentFile.getState() == UploadFile.UPLOADED)
			{
				fileList.add(currentFile);
			}
			
		}
		
		return fileList;
	}
	
	/**
	 * Retourne la liste des fichiers annul par l'utilisateur
	 * @return la liste des fichiers annul par l'utilisateur
	 */
	public Vector<UploadFile> getVectFileListCancelled()
	{
		Vector<UploadFile> fileList = new Vector<UploadFile>(); 
		for (UploadFile currentFile : this.vectFileList)
		{
			if(currentFile.getState() == UploadFile.CANCELLED)
			{
				fileList.add(currentFile);
			}
			
		}
		
		return fileList;
	}
}


