/**
 * 
 */
package dluploadclient;

import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Boite de dialogue pour l'affichage des URL de l'application dl.free.fr
 * 
 * <p>
 * <code>
 * Copyright 2008, Maurer Gilles
 * </code>
 * <p>
 * <code>
 * This file is part of Client dl.free.fr.
 * </code>
 * <p>
 * <code>
 * Client dl.free.fr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * </code>
 *<p>
 * <code>
 * Client dl.free.fr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * </code>
 * <p>
 * <code>
 * You should have received a copy of the GNU General Public License
 * along with Client dl.free.fr. If not, see <http://www.gnu.org/licenses/>.
 * </code>
 * 
 * @author Gilles Maurer
 *
 */
public class DLUploadClientURLGUI extends JDialog
{
	/**
	 * Numro de srie pour la srialisation
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Rfrence  la fentre parente
	 */
	private DLUploadClientGUI gui;
	
	/**
	 * Composant pour l'affichage des liens
	 */
	private JTextArea jTxtAreaContent;

	/**
	 * Liste de choix pour choisir le type d'affichage
	 */
	private JComboBox jCmbBoxMode;
	
	/**
	 * Liste des diffrents modes d'affichage
	 */
	private String[] listMode;
	
	/**
	 * Bouton de copie du contenu de la zone de texte
	 */
	private JButton jBtnCopyToClipboard;
	
	/**
	 * Affichage du texte "Mode :"
	 */
	private JLabel jLblMode;
	
	
	
	/**
	 * Constante pour dfinir le mode Texte
	 */
	public static final String TEXT = "Texte";
	/**
	 * Constante pour dfinir le mode Texte (URL uniquement)
	 */
	public static final String TEXTURLONLY = "Texte (URL uniquement)";
	/**
	 * Constante pour dfinir le mode HTML
	 */
	public static final String HTML = "HTML";
	/**
	 * Constante pour dfinir le mode Forum phpBB (BB code)
	 */
	public static final String PHPBB = "Forum phpBB (BB code)";
	
	/**
	 * Sous titre de la fentre
	 */
	public static final String WINDOWTITLE = "Liste de liens";

	/**
	 * Constructeur 
	 * @param gui Rfrence  la fentre parente
	 * @param owner Rfrence  la Frame parente
	 * @param modal Vrai si la fentre doit tre modale, faux sinon.
	 */
	public DLUploadClientURLGUI(DLUploadClientGUI gui, Frame owner, boolean modal)
	{
		// On utilise le constructeur de la classe JDialog pour rendre la fentre modale, ou non
		super(owner, modal);
		this.gui = gui;
		
		// On dfinit les paramtres de la boite de dialogue
		this.setTitle(DLUploadClientGUI.WINDOWTITLE + ' ' + DLUploadClientURLGUI.WINDOWTITLE);
		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		
		// Cration des composant de l'interface
		this.jTxtAreaContent = new JTextArea(20,80);
		this.jTxtAreaContent.setEditable(false);
		this.jBtnCopyToClipboard = new JButton("Copier");
		this.jLblMode = new JLabel("Mode : ");
		
		this.listMode = new String[4];
		
		this.listMode[0] = DLUploadClientURLGUI.TEXT;
		this.listMode[1] = DLUploadClientURLGUI.TEXTURLONLY;
		this.listMode[2] = DLUploadClientURLGUI.HTML;
		this.listMode[3] = DLUploadClientURLGUI.PHPBB;
		
		this.jCmbBoxMode = new JComboBox(this.listMode);
		
		// Ajout des vnements
		this.jCmbBoxMode.addItemListener(new JCmbBoxModeItemListener(this));
		this.jBtnCopyToClipboard.addActionListener(new JBtnCopyToClipboardListener(this));
		
		// Ajout du layout
		Container contentPane = this.getContentPane();
		GridBagLayout backgroundLayout = new GridBagLayout();
		contentPane.setLayout(backgroundLayout);
		
		JScrollPane jTxtAreasSrollPane = new JScrollPane(this.jTxtAreaContent); 		

		// Ajout des composants
		Insets globalInsets = new Insets(5, 5, 5, 5);
		
		GridBagConstraints constraint = new GridBagConstraints();
		constraint.gridx = 1;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 1;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		constraint.anchor = GridBagConstraints.WEST;
		contentPane.add(this.jCmbBoxMode, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 0;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		constraint.anchor = GridBagConstraints.WEST;
		contentPane.add(this.jLblMode, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 1;
		constraint.gridwidth = GridBagConstraints.REMAINDER;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = DLUploadClientGUI.WEIGHTX_BORDER;
		constraint.weighty = DLUploadClientGUI.WEIGHTY_NORMAL;
		constraint.fill = GridBagConstraints.BOTH;
		contentPane.add(jTxtAreasSrollPane, constraint);
		
		constraint = new GridBagConstraints();
		constraint.gridx = 0;
		constraint.gridy = 2;
		constraint.ipadx = DLUploadClientGUI.VERTICAL_PADDING;
		constraint.ipady = DLUploadClientGUI.HORIZONTAL_PADDING;
		constraint.insets = globalInsets;
		constraint.weightx = 0;
		constraint.weighty = 0;
		constraint.fill = GridBagConstraints.NONE;
		constraint.anchor = GridBagConstraints.NORTHWEST;
		contentPane.add(this.jBtnCopyToClipboard, constraint);
		
		// On lance automatiquemment l'affichage du text brut
		this.generateContent(DLUploadClientURLGUI.TEXT);
	}
	
	/**
	 * Met  jour le texte de la fentre
	 * @param text Texte  mettre en place
	 */
	public void setContent(String text)
	{
		this.jTxtAreaContent.setText(text);
	}
	
	/**
	 * Gnre le texte de la fentre selon le mode choisis
	 * @param mode Le mode choisi pour gnrer le contenu 
	 */
	public void generateContent(String mode)
	{
		String text = new String("");
		String format = new String("");
		Vector<UploadFile> files = this.gui.getVectFileListUploaded();
		
		/*
		 * Format du template d'affichage :
		 * %1$s : L'URL du fichier
		 * %2$s : Le Nom de fichier
		 */
		
		if(mode.equals(DLUploadClientURLGUI.TEXT))
		{
			format = "%2$s : %1$s";
		}
		else if(mode.equals(DLUploadClientURLGUI.TEXTURLONLY))
		{
			format = "%1$s";
		}
		else if(mode.equals(DLUploadClientURLGUI.HTML))
		{
			format = "<a href=\"%1$s\">%2$s</a><br />";
		}
		else if(mode.equals(DLUploadClientURLGUI.PHPBB))
		{
			format = "[url=%1$s]%2$s[/url]";
		}
		
		for (UploadFile uploadFile : files)
		{
			text += String.format(format, uploadFile.getUrl(), uploadFile.getLocalFilename()) + "\n";
		}
		this.setContent(text);
	}
	
	/**
	 * Retourne le contenu du texte de la fentre
	 * @return le contenu du texte de la fentre
	 */
	public String getContent()
	{
		return this.jTxtAreaContent.getText();
	}
	
}
