/**
 * 
 */
package dluploadclient;

import java.math.BigDecimal;
import java.math.RoundingMode;

import com.enterprisedt.net.ftp.FTPProgressMonitor;

/**
 * Classe qui suit la progression de l'upload en FTP
 * 
 * <p>
 * <code>
 * Copyright 2008, Maurer Gilles
 * </code>
 * <p>
 * <code>
 * This file is part of Client dl.free.fr.
 * </code>
 * <p>
 * <code>
 * Client dl.free.fr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * </code>
 *<p>
 * <code>
 * Client dl.free.fr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * </code>
 * <p>
 * <code>
 * You should have received a copy of the GNU General Public License
 * along with Client dl.free.fr. If not, see <http://www.gnu.org/licenses/>.
 * </code>
 * 
 * @author Maurer Gilles
 *
 */
public class DLUploadProgressMonitor implements FTPProgressMonitor
{
	/**
	 * Rfrence  la fentre principale de l'application
	 */
	private DLUploadClientGUI gui;
	
	/**
	 * Taille du fichier actuel
	 */
	private long filesize;
	
	/**
	 * Rfrence vers le fichier en cours d'upload 
	 */
	private UploadFile currentUploadFile;
	
	/**
	 * Rfrence vers le thread d'upload 
	 */
	private ThreadUpload thread;
	
	
	/**
	 * Constructeur par dfaut
	 * @param gui Rfrence  la fentre principale de l'application
	 * @param filesize Taille du fichier actuel
	 * @param uploadFile Rfrence vers le fichier en cours d'upload
	 */
	public DLUploadProgressMonitor(DLUploadClientGUI gui, long filesize, UploadFile uploadFile, ThreadUpload thread)
	{
		this.gui = gui;
		this.filesize = filesize;
		this.currentUploadFile = uploadFile;
		this.thread = thread;
	}
	/* (non-Javadoc)
	 * @see com.enterprisedt.net.ftp.FTPProgressMonitor#bytesTransferred(long)
	 */
	public void bytesTransferred(long position)
	{
		
		// Le code pour le fichier courant
		double currentValue = ((double) (position + this.gui.getRestartSize()) / (double)this.filesize)* 100;
		this.gui.setCurrentUploadProgressBarValue(currentValue);
		this.currentUploadFile.setPercentDone((float)currentValue);
		
		// Le code pour la totalit des transferts
		BigDecimal totalPercentUploaded = new BigDecimal("0");
		totalPercentUploaded = this.thread.getCurrentSizeUploaded().add(BigDecimal.valueOf(position + this.gui.getRestartSize()));
		totalPercentUploaded = totalPercentUploaded.divide(this.thread.getTotalSizeToUpload(),10, RoundingMode.HALF_EVEN);
		totalPercentUploaded = totalPercentUploaded.multiply(BigDecimal.valueOf(100));
		this.gui.setGlobalProgressBarValue(totalPercentUploaded.floatValue());
		
		this.gui.jtableFileListRefresh();
		
	}

}
