package dluploadclient;
import java.io.File;
import java.io.FileInputStream;
import java.math.BigDecimal;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.enterprisedt.net.ftp.FTPClient;
import com.enterprisedt.net.ftp.FTPTransferType;



/**
 * Cette classe permet de crer un processus lger qui permet d'uploader les fichiers
 * 
 * <p>
 * <code>
 * Copyright 2008, Maurer Gilles
 * </code>
 * <p>
 * <code>
 * This file is part of Client dl.free.fr.
 * </code>
 * <p>
 * <code>
 * Client dl.free.fr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * </code>
 *<p>
 * <code>
 * Client dl.free.fr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * </code>
 * <p>
 * <code>
 * You should have received a copy of the GNU General Public License
 * along with Client dl.free.fr. If not, see <http://www.gnu.org/licenses/>.
 * </code>
 * 
 * @author Maurer Gilles
 *
 */
public class ThreadUpload extends Thread
{
	/**
	 * Rfrence  la fentre principale
	 */
	private DLUploadClientGUI gui;
	
	/**
	 * Le client FTP
	 */
	private FTPClient ftp;
	
	/**
	 * Le nom d'utilisateur
	 */
	private String user;
	
	/**
	 * Le mot de passe
	 */
	private String password;
	
	/**
	 * Permet d'indiquer  l'application que l'utilisateur a dcider de mettre fin  l'upload
	 */
	private boolean cancelUpload;
	
	/**
	 * La taille totale des fichiers  uploader
	 */
	private BigDecimal totalSizeToUpload;
	
	/**
	 * La taille totale des fichiers dja compltement upload
	 */
	private BigDecimal currentSizeUploaded;
	
	
	/**
	 * Construteur par dfaut
	 * @param gui Rfrence  la fentre principale
	 */
	public ThreadUpload(DLUploadClientGUI gui)
	{
		this.gui = gui;
		this.totalSizeToUpload = new BigDecimal("0");
		this.currentSizeUploaded = new BigDecimal("0");
		
	}

	/* (non-Javadoc)
	 * @see java.lang.Thread#run()
	 */
	public void run()
	{
		// Mise  jour de l'tat de l'application
		this.gui.setApplicationState(DLUploadClientGUI.UPLOADING);
		this.gui.launchEventAndStatusManager();
		
		// On cre une file de fichier  uploader  ; Et on ne prends que ceux qui ne sont pas encore upload !
		LinkedList<UploadFile> uploadFiles = new LinkedList<UploadFile>(this.gui.getVectFileListToUpload());
		
		// On calcule la taille totale  transfrer
		this.getTotalBytesToTransfer();
		
		// On cre les variables utilises lors de l'upload de chaque fichier
		UploadFile currentUploadFile;
		File currentFile;
		String fileName;
		long restartSize = 0;
		this.cancelUpload = false;
		boolean isResume = true;
		boolean isCounted = false;
		
		// S'il reste des fichiers et que l'utilisateur n'a pas annul l'upload 
		while(!uploadFiles.isEmpty() && !this.cancelUpload)
		{
			// On indique que la taille n'a pas encore t compte
			isCounted = false;
			
			// On rcupre le fichier courant  uploader
			currentUploadFile = uploadFiles.removeFirst();
			
			// On ouvre le fichier
			currentFile = new File(currentUploadFile.getPath());
			
			// On cre un nom de fichier sans caratre spciaux
			fileName = DLUploadClientGUI.sanitizeFilename(currentFile.getName());
			
			// On modifie le statut du fichier, pour mettre  jour la liste de fichiers !
			currentUploadFile.setState(UploadFile.UPLOADING);
			
			// Permet de lire les commandes et retour du serveur
			FTPMessageDisplay listener = new FTPMessageDisplay(this.gui);

			// Log sur la console
			/*Logger log = Logger.getLogger(ThreadUpload.class);
        	Logger.setLevel(Level.ALL);*/

			try
			{
				// Cration du stream d'entr
				FileInputStream fileInput = new FileInputStream(currentFile);

				// On cre le client FTP
				this.ftp = new FTPClient();
				// On enregistre les transactions
				this.ftp.setMessageListener(listener);

				try
				{
					// On informe l'utilisateur qu'on se connecte
					listener.logInfo("Connexion  dl.free.fr");
					
					// On se logge
					this.ftp.setRemoteHost("dl.free.fr");
					this.ftp.setRemotePort(21);
					this.ftp.connect();
					this.ftp.login(this.user, this.password);
					
					// On informe l'utilisateur que la connexion a russie
					listener.logInfo("Connexion tablie  dl.free.fr");

					// On se place en mode Binaire
					this.ftp.setType(FTPTransferType.BINARY);
					// On met en place le moniteur pour la barre de progression
					this.ftp.setProgressMonitor(new DLUploadProgressMonitor(this.gui, currentFile.length(), currentUploadFile, this), 2048);

					// On informe l'utilisateur qu'on vrifie l'existence du fichier
					listener.logInfo("Vrification de l'existence du fichier");
					
					// On tente de demander la taille du fichier
					try
					{
						restartSize = this.ftp.size(fileName);
					}
					catch (Exception e)
					{
						// S'il y a une erreur, on prsuppose que le resume ne fonctionnera pas ; On dsactive le resume et on commence  transfrer  partir de l'octet 0.
						isResume = false;
						restartSize = 0;
					}

					if(isResume)
					{
						this.ftp.resume();
					}

					// On met  jour la taille "actuelle" du fichier s'il existe
					this.gui.setRestartSize(restartSize);

					// On informe l'utilisateur qu'on lance l'envoi du fichier
					listener.logInfo("Dmarrage de l'envoi du fichier");

					// On envoie le fichier en rsume, ou non.
					this.ftp.put(fileInput, fileName, isResume);
					
					// On informe l'utilisateur que l'envoi du fichier est termin
					listener.logInfo("Envoi du fichier termin");

					// On ajoute le fait que le fichier soit transfr
					this.currentSizeUploaded = this.currentSizeUploaded.add(BigDecimal.valueOf(currentUploadFile.getSize()));
					isCounted = true;

					// On rcupre les rponses du serveur FTP aprs l'envoi, pour rcuprer l'URL du fichier
					String[] ftpReplys = this.ftp.getLastValidReply().getReplyData();
					Pattern p = Pattern.compile(".*(http://dl.free.fr/.*$)");

					String url = new String();

					// On recherche la dernire URL contenue dans le log FTP.
					for (String ftpReply : ftpReplys)
					{
						Matcher m = p.matcher(ftpReply);
						if(m.matches())
						{
							url = m.group(1);
						}
					}

					// On test si on a trouv quelquechose qui ressemble  une URL, ou non :-)
					Matcher m = p.matcher(url);
					if(!m.matches())
					{
						url = "Impossible de rcuprer l'adresse du fichier.";
					}

					// On met  jour l'URL dans la liste des fichiers (et on rajoute le nom de fichier sans caractre spciaux, pour que ce soit plus joli)
					currentUploadFile.setUrl(url + "/" + fileName);
					
					// On met  jour l'tat du fichier
					// Si l'annulation a t demande
					if(this.cancelUpload)
					{
						currentUploadFile.setState(UploadFile.CANCELLED);
					}
					// Si l'upload est termin
					else
					{
						currentUploadFile.setState(UploadFile.UPLOADED);
					}

					// On informe l'utilisateur qu'on ferme la connexion
					listener.logInfo("Fermeture de la connexion  dl.free.fr");

					// On ferme la connexion
					this.ftp.quit();
					
					// On informe l'utilisateur que la connexion est ferme
					listener.logInfo("Connexion  dl.free.fr ferme");					

					// On remet  zero la barre de progression
					this.gui.setCurrentUploadProgressBarValue(0);
					
				}
				// Si quelquechose a chou, pour une raison X ou Y
				catch (Exception e)
				{					
					// On met  jour le message d'erreur correspondant
					currentUploadFile.setErrorMessage(e.getMessage());
					
					// Et surtout, on rajoute la taille du fichier pour la barre de progression au cas o ce ne serais pas dj fait
					if(!isCounted)
					{
						this.currentSizeUploaded = this.currentSizeUploaded.add(BigDecimal.valueOf(currentUploadFile.getSize()));
					}
					
					this.ftp.quit();
				}
			}
			// Si quelquechose a chou, pour une raison X ou Y
			catch (Exception e)
			{				
				// On met  jour le message d'erreur correspondant
				currentUploadFile.setErrorMessage(e.getMessage());
				
				this.currentSizeUploaded = this.currentSizeUploaded.add(BigDecimal.valueOf(currentUploadFile.getSize()));
			}
		}
		
		// On met  jour le statut de l'application
		this.gui.setApplicationState(DLUploadClientGUI.WAITING);
		this.gui.launchEventAndStatusManager();
	}
	
	/**
	 * Permet de mettre  jour l'utilisateur FTP
	 * @param user Utilisateur FTP  utiliser
	 */
	public void setUser(String user)
	{
		this.user = user;
	}
	
	/**
	 * Permet de mettre  jour le mot de passe 
	 * @param password Mot de passe FTP  utiliser
	 */

	public void setPassword(String password)
	{
		this.password = password;
	}
	
	/**
	 * Permet d'accder au client FTP depuis l'extrieur de la classe
	 * @return Une rfrence vers le client FTP
	 */
	public FTPClient getFTPClient()
	{
		return this.ftp;
	}
	
	public void cancelUpload()
	{
		// On remonte au thread que l'utilisateur veut annuler le transfert
		this.cancelUpload = true;
	}
	
	private void getTotalBytesToTransfer()
	{
		UploadFile currentUploadFile;
		LinkedList<UploadFile> uploadFiles= new LinkedList<UploadFile>(this.gui.getVectFileListToUpload());
		while(!uploadFiles.isEmpty())
		{
			// On rcupre le fichier courant  uploader
			currentUploadFile = uploadFiles.removeFirst();
			
			// On ajoute sa taille  la taille totale
			this.totalSizeToUpload = this.totalSizeToUpload.add(BigDecimal.valueOf(currentUploadFile.getSize()));
		}
	}

	/**
	 * Retourne la taille totale  uploader
	 * @return la taille totale  uploader
	 */
	public BigDecimal getTotalSizeToUpload()
	{
		return totalSizeToUpload;
	}

	/**
	 * Retourne la taille totale des fichiers dja compltement upload
	 * @return la taille totale des fichiers dja compltement upload
	 */
	public BigDecimal getCurrentSizeUploaded()
	{
		return currentSizeUploaded;
	}
}
